<?xml version="1.0" encoding="utf-8"?>
<!-- ============================================================== -->
<!-- This transform generates the narrative block of an HAI
     CDA document. Aside from that, it's an identity transform.     -->
<!-- ============================================================== -->
<!-- Created by Alschuler Associates, LLC, February 2008
     Updated February 2009 for R3.
     Updated November 2009 for R4
     Updated January  20010 for R4 update
                                                                    -->
<!-- ============================================================== -->
<!-- ============================================================== -->
<!-- The transform consists of five segments:

     o Main segment
         - Default handling (identity transform)
         - Top-level sections
            - creates table for the rows of data in the narrative block
            - enters generate-narrative mode to process the entries
              into the narrative block

      o Entries segment
          - process-children in mode generate-narrative
          - Routing template

      o Patterns segment
          - Processing patterns
          - Exceptions templates

      o CDA Non-templated elements
          - e.g., methodCode; effectiveTime in some contexts

      o Utilities segment
         - CDA Utilities (e.g., get-displayName)
         - General utilities (format-date)

     The approach is detailed below.
-->
<!-- The approach of this transform is as follows:

     a) An identity transform. This puts all the original header
        and entries into the output file.

     b) At the beginning of each CDA section, an interruption
        to create the narrative block: each entry is processed
        in generate-narrative mode.

        - The default behavior for generate-narrative mode is to
          process-children in generate-narrative mode. Thus each
          element node of an entry is touched.

        - In generate-narrative mode, a templated node (a clinical statement
          in an entry) is routed to one or another pattern-handling template,
          or to an exception template.

          (Except for organizers and encounters, which are templated but
           often provide no data for the display.)

        - A templateId which is not handled explicitly by the routing
          will kick out a diagnostic showing the templateId/@root value.

        The patterns and exceptions templates produce a label and
        datum, and (usually) process-children so that any
        untemplated subelements (such as methodCode or effectiveTime)
        are touched - and so that any templated elements will be
        encountered. Templated elements, of course, are then caught by
        the routing template.

     Thus, when adding a new templateId,

           a) Determine whether its display fits one of the
              existing patterns, requires a new pattern, or
              is an exception

           b) Route it appropriately in the routing template

           c) If a new pattern, create the pattern template.
              If an exception, create an exception template.

           d) If any untemplated subelements (beyond code and value)
              need to be displayed, examine the "Untemplated Elements"
              section of this transform to see how they will be handled,
              and modify as needed.
-->
<!-- ============================================================== -->
<!--                    TRANSFORM                                   -->
<!-- ============================================================== -->
<!-- ============================================================== -->
<xsl:transform version="1.1" 
               xmlns="urn:hl7-org:v3" 
               xmlns:cda="urn:hl7-org:v3" 
               xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
               xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" 
               exclude-result-prefixes="cda">
  <xsl:output method="xml" indent="yes" encoding="UTF-8"/>
  <!-- Note: Does not strip space in ClinicalDocument, structuredBody, component,
     or section because in the sample document there are linespaces to help
     readability. -->
  <xsl:strip-space elements="
            cda:recordTarget cda:patientRole cda:patient cda:name
            cda:author cda:assignedAuthor
            cda:custodian cda:assignedCustodian cda:representedCustodianOrganization
            cda:legalAuthenticator cda:assignedEntity
            cda:componentOf cda:encompassingEncounter cda:effectiveTime
            cda:location cda:healthCareFacility cda:addr
            cda:act cda:procedure cda:observation
            cda:entry cda:organizer cda:entryRelationship cda:component"/>
  <!-- ============================================================== -->
  <!--                    INCLUDES and RESOURCES                      -->
  <!-- ============================================================== -->
  <!-- Vocabulary handling -->
  <!-- read voc.xml into a lookup table -->
  <xsl:variable name="the-valuesets" select="document('hai_voc.xml')"/>
  <!-- ============================================================== -->
  <!-- ============================================================== -->
  <!--                    MAIN SEGMENT                                -->
  <!-- ============================================================== -->
  <!-- ============================================================== -->
  <!-- ============================================================== -->
  <!--                    DEFAULT HANDLING: IDENTITY TRANSFORM        -->
  <!-- ============================================================== -->
  <!-- This template adds typeCode='DRIV' to the entry. This means
     that the corresponding information in the narrative block is
     derived entirely from the entry, which is true.

     If a later user interface allows entry directly into the
     narrative block, change this template accordingly. -->
  <xsl:template match="*|@*|comment()|processing-instruction()|text()">
    <xsl:copy>
      <xsl:if test="self::cda:entry">
        <xsl:attribute name="typeCode">DRIV</xsl:attribute>
      </xsl:if>
      <xsl:apply-templates select="*|@*|comment()|processing-instruction()|text()"/>
    </xsl:copy>
  </xsl:template>
  <!-- ============================================================== -->
  <!--                    TOP-LEVEL SECTIONS                          -->
  <!-- ============================================================== -->
  <!-- These CDA documents have no nested sections. -->
  <xsl:template match="/cda:ClinicalDocument/cda:component/cda:structuredBody
                     /cda:component/cda:section">
    <xsl:copy>
      <xsl:call-template name="start-section"/>
      <text>
        <table>
          <tbody>
            <!-- process entries to create narrative block -->
            <xsl:apply-templates select="cda:entry" mode="generate-narrative"/>
          </tbody>
        </table>
      </text>
      <!-- back to the identity transform -->
      <xsl:apply-templates select="*[not(self::cda:title
                                     | self::cda:code
                                     | self::cda:templateId
                                     | self::cda:text)]"/>
    </xsl:copy>
  </xsl:template>
  <!-- named template: start-section -->
  <xsl:template name="start-section">
    <xsl:copy-of select="cda:templateId"/>
    <xsl:copy-of select="cda:code"/>
    <!-- section/title is required -->
    <title>
      <xsl:value-of select="cda:title"/>
    </title>
  </xsl:template>
  <!-- ============================================================== -->
  <!-- ============================================================== -->
  <!--                    ENTRIES SEGMENT                             -->
  <!-- ============================================================== -->
  <!-- ============================================================== -->
  <!-- This template ensures that by default we stay in generate-
     narrative mode while creating the text block. -->
  <xsl:template match="*" mode="generate-narrative">
    <xsl:apply-templates select="*" mode="generate-narrative"/>
  </xsl:template>
  <!-- ============================================================== -->
  <!--                    TEMPLATED ELEMENTS: ROUTING TO PATTERNS     -->
  <!-- ============================================================== -->
  <!-- The routing template sends a clinical statement to 
       the appropriate type of display handling. -->
  <xsl:template match="*[not(self::ClinicalDocument) and not(self::cda:section)][cda:templateId]" mode="generate-narrative">
    <xsl:choose>
      <!-- 1. Displayed value is Yes/No.
                Label = process value/@code (=get @displayName from voc.xml)
                Value = from @negationInd  
      -->
      <xsl:when test="cda:templateId/@root='2.16.840.1.113883.10.20.5.2.1.1.1'
                   or cda:templateId/@root='2.16.840.1.113883.10.20.5.2.2.7.5'
                   or cda:templateId/@root='2.16.840.1.113883.10.20.5.2.2.7.6'
                   or cda:templateId/@root='2.16.840.1.113883.10.20.5.2.2.7.7'
                   or cda:templateId/@root='2.16.840.1.113883.10.20.5.2.2.7.15'
                   or cda:templateId/@root='2.16.840.1.113883.10.20.5.2.5.2'
                   or cda:templateId/@root='2.16.840.1.113883.10.20.5.6.5'
                   or cda:templateId/@root='2.16.840.1.113883.10.20.5.6.12'
                   or cda:templateId/@root='2.16.840.1.113883.10.20.5.6.19'
                   or cda:templateId/@root='2.16.840.1.113883.10.20.5.6.20'              
                   or cda:templateId/@root='2.16.840.1.113883.10.20.5.6.24'
                   or cda:templateId/@root='2.16.840.1.113883.10.20.5.6.25'
                   or cda:templateId/@root='2.16.840.1.113883.10.20.5.6.29'
                   or cda:templateId/@root='2.16.840.1.113883.10.20.5.6.31'
                   or cda:templateId/@root='2.16.840.1.113883.10.20.5.6.37'
                   or cda:templateId/@root='2.16.840.1.113883.10.20.5.6.39'
                   or cda:templateId/@root='2.16.840.1.113883.10.20.5.6.44'
                   or cda:templateId/@root='2.16.840.1.113883.10.20.5.6.48'
                   or cda:templateId/@root='2.16.840.1.113883.10.20.5.6.50'
                   or cda:templateId/@root='2.16.840.1.113883.10.20.5.6.55'
                 ">
        <xsl:call-template name="pattern-1"/>
      </xsl:when>
      
      <!-- 2. Label from value/@codeSystem (usu. NHSN)
                Label = process value/@codeSystem with lookup in voc.xml if it's NHSN
                Value = process value element
      -->   
      <xsl:when test="cda:templateId/@root='2.16.840.1.113883.10.20.5.2.2.7.8'">
        <xsl:call-template name="pattern-2"/>
      </xsl:when>
      
      <!-- 3. Label from templateId
                Label = map the templateId to a label
                Value = process the value element
         5.2.2.3 and 5.2.2.3.1 were removed (LCBI pathway)
      -->
      <xsl:when test="cda:templateId/@root='2.16.840.1.113883.10.20.5.2.2.7.4'
                   or cda:templateId/@root='2.16.840.1.113883.10.20.5.2.5.1'
                   or cda:templateId/@root='2.16.840.1.113883.10.20.5.6.4'
                   or cda:templateId/@root='2.16.840.1.113883.10.20.5.6.21'
                   or cda:templateId/@root='2.16.840.1.113883.10.20.5.6.23'
                   or cda:templateId/@root='2.16.840.1.113883.10.20.5.6.27'
                   or cda:templateId/@root='2.16.840.1.113883.10.20.5.6.49'
                   or cda:templateId/@root='2.16.840.1.113883.10.20.5.6.56'
                 ">
        <xsl:call-template name="pattern-3"/>
      </xsl:when>
      
      <!-- 4. Question/answer pattern: 
                 Label = process the code element
                 Value = process the value element 
      -->
      <xsl:when test="cda:templateId/@root='2.16.840.1.113883.10.20.5.2.1.2'
                   or cda:templateId/@root='2.16.840.1.113883.10.20.5.2.2.7.9'
                   or cda:templateId/@root='2.16.840.1.113883.10.20.5.2.2.7.10'
                   or cda:templateId/@root='2.16.840.1.113883.10.20.5.2.2.7.11'
                   or cda:templateId/@root='2.16.840.1.113883.10.20.5.2.2.7.12'
                   or cda:templateId/@root='2.16.840.1.113883.10.20.5.2.5.1.2'
                   or cda:templateId/@root='2.16.840.1.113883.10.20.5.2.1.1.2'
                   or cda:templateId/@root='2.16.840.1.113883.10.20.5.6.26'
                   or cda:templateId/@root='2.16.840.1.113883.10.20.5.6.46'
                   or cda:templateId/@root='2.16.840.1.113883.10.20.5.6.52'
      ">
        <xsl:call-template name="pattern-4"/>
      </xsl:when>
      
      <!-- 5. Code/negation pattern: 
                 Label = process code
                 Value = process negationInd
      -->
      <xsl:when test="cda:templateId/@root='2.16.840.1.113883.10.20.5.2.1.3'
                   or cda:templateId/@root='2.16.840.1.113883.10.20.5.6.16'
                   or cda:templateId/@root='2.16.840.1.113883.10.20.5.6.15'
                 ">
        <xsl:call-template name="pattern-5"/>
      </xsl:when>
      
      <!-- Not-patterned (exception) handling -->
      <xsl:when test="cda:templateId/@root='2.16.840.1.113883.10.20.5.2.2.7.2'
                   or cda:templateId/@root='2.16.840.1.113883.10.20.5.2.2.7.3'
                   or cda:templateId/@root='2.16.840.1.113883.10.20.5.2.5.1.1'
                   or cda:templateId/@root='2.16.840.1.113883.10.20.5.6.1'
                   or cda:templateId/@root='2.16.840.1.113883.10.20.5.6.2'
                   or cda:templateId/@root='2.16.840.1.113883.10.20.5.6.3'
                   or cda:templateId/@root='2.16.840.1.113883.10.20.5.6.6'
                   or cda:templateId/@root='2.16.840.1.113883.10.20.5.6.7'
                   or cda:templateId/@root='2.16.840.1.113883.10.20.5.6.10'
                   or cda:templateId/@root='2.16.840.1.113883.10.20.5.6.11'
                   or cda:templateId/@root='2.16.840.1.113883.10.20.5.6.13'
                   or cda:templateId/@root='2.16.840.1.113883.10.20.5.6.17'
                   or cda:templateId/@root='2.16.840.1.113883.10.20.5.6.18'
                   or cda:templateId/@root='2.16.840.1.113883.10.20.5.6.22'
                   or cda:templateId/@root='2.16.840.1.113883.10.20.5.6.32'
                   or cda:templateId/@root='2.16.840.1.113883.10.20.5.6.33'
                   or cda:templateId/@root='2.16.840.1.113883.10.20.5.6.34'
                   or cda:templateId/@root='2.16.840.1.113883.10.20.5.6.35'
                   or cda:templateId/@root='2.16.840.1.113883.10.20.5.6.36'
                   or cda:templateId/@root='2.16.840.1.113883.10.20.5.6.38'
                   or cda:templateId/@root='2.16.840.1.113883.10.20.5.6.40'
                   or cda:templateId/@root='2.16.840.1.113883.10.20.5.6.41'
                   or cda:templateId/@root='2.16.840.1.113883.10.20.5.6.43'
                   or cda:templateId/@root='2.16.840.1.113883.10.20.5.6.45'
                   or cda:templateId/@root='2.16.840.1.113883.10.20.5.6.53'
                   or cda:templateId/@root='2.16.840.1.113883.10.20.5.6.54'
                   or cda:templateId/@root='2.16.840.1.113883.10.20.1.21'
                   or cda:templateId/@root='2.16.840.1.113883.10.20.5.6.47'">
        <xsl:apply-templates select="." mode="exceptions"/>
      </xsl:when>

      <!-- By default organizers and encounters are processed silently,
           since they usually provide no data for the display. To process
           one differently, list it above as an exception pattern. -->
      <xsl:when test="self::cda:organizer | self::cda:encounter">
        <xsl:apply-templates select="*" mode="generate-narrative"/>
      </xsl:when>

      <!-- For a few we explicitly do nothing:
            ...20.5.6.42 is handled by ...20.5.6.43 
            ...20.5.6.13 is handled by ...20.5.6.18
            ...20.5.6.30 is handled by ...20.5.6.18
            ...20.5.6.28 is handled by ...20.5.6.18
            ...20.5.6.9 is handled by  ...20.5.6.18
            
            ...20.1.53 is a consumable, do not treat its templateId
                                        as indicating a clinical statement -->
      <xsl:when test="cda:templateId/@root='2.16.840.1.113883.10.20.5.6.42'
                   or cda:templateId/@root='2.16.840.1.113883.10.20.5.6.9'
                   or cda:templateId/@root='2.16.840.1.113883.10.20.5.6.28'
                   or cda:templateId/@root='2.16.840.1.113883.10.20.5.6.13'
                   or cda:templateId/@root='2.16.840.1.113883.10.20.1.53'"/>

      <!-- default = diagnostic -->
      <xsl:otherwise>
        <tr>
          <td valign="top">[[<xsl:value-of select="cda:templateId/@root"/>]]
          <xsl:value-of select="local-name(.)"/>
          </td>
          <td valign="top"/>
        </tr>
        <xsl:apply-templates select="*" mode="generate-narrative"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <!-- By default organizers and encounters are passed through silently,
       since they usually provide no data for the display. -->
  <!--
  <xsl:template match="cda:organizer | cda:encounter" mode="generate-narrative" priority="2">
    <xsl:apply-templates select="*" mode="generate-narrative"/>
  </xsl:template>
-->
  <!-- ============================================================== -->
  <!-- ============================================================== -->
  <!--                    PATTERNS SEGMENT                            -->
  <!-- ============================================================== -->
  <!-- ============================================================== -->
  <!-- The data are presented in the narrative block in two-column
     rows where the cells are: field label, field datum.

     Source data comes from different child elements depending on the
     structure of the clinical statement and on what is wanted
     for display.

     This transform makes use of five display patterns - and a set of
     exceptions. The patterns are:

       1. Assertion, create label from value element, display is yes/no, 

                Label = process value/@code (=get @displayName from voc.xml)
                Value = from @negationInd  

       2. Assertion, create the label from codeSystem (usu. NHSN)

                Label = process value/@codeSystem
                Value = process value/@code

       3. Assertion, create the label from templateId

                Label = map the templateId to a label
                Value = process value/@code

       4. Question/answer pattern, create label from code, value from value element

                 Label = process the code element
                 Value = process the value element, with some attention to @xsi:type 
                          (default is process value/@code)

       5. Code/Negation pattern
       
                 Label = process the code element
                 Value = process negationInd

      The exceptions handle:

       a) Design inconsistencies that currently prevent the
          use of one of the above patterns

       b) Minor variations that would be elevated to a pattern
          if they occurred more often

       c) Procedures, which only need to process-children.
          This template is technically superfluous. However,
          the transform intentionally kicks out a diagnostic
          for any templateId that has not been handled explicitly;
          this procedures template circumvents that.

       Where appropriate (which is almost always), the patterns
       and exceptions templates also process-children. This ensures
       that any child entryRelationships or organizers are processed
       and their templated children passed back to the routing
       template.

       By default in mode=generate-narrative, any untemplated element is
       passed over silently. So, any untemplated element that is to
       produce output for display must be handled explicitly. These
       templates are in the section "Untemplated Elements". For
       example, methodCode and approachSiteCode are handled there,
       and so is effectiveTime when in an Occasion of HAI Observation,
       (which is one of the few places we want to display effectiveTime).
-->
  <!-- ============================================================== -->
  <!--                    OBSERVATION: ASSERTION PATTERN 1            -->
  <!-- ============================================================== -->
  <!--  1. Yes/no question
           Label = process value/@code (=get @displayName from voc.xml)
           Value = from @negationInd  
-->
  <!-- context node is the clinical statement -->
  <xsl:template name="pattern-1">
    <tr>
      <td valign="top">
        <!-- change context node -->
        <xsl:for-each select="cda:value">
          <xsl:call-template name="get-displayName"/>
        </xsl:for-each>
      </td>
      <td valign="top">
        <xsl:call-template name="get-truthValue"/>
      </td>
    </tr>
    <xsl:apply-templates select="*" mode="generate-narrative"/>
  </xsl:template>
  <!-- ============================================================== -->
  <!--                    OBSERVATION: ASSERTION PATTERN 2            -->
  <!-- ============================================================== -->
  <!--
       2. Assertion, create the label from codeSystem (usu. NHSN)

          Label = process value/@codeSystem
          Value = process value/@code 
-->
  <!-- context node is the clinical statement -->
  <xsl:template name="pattern-2">
    <tr>
      <!-- change context node -->
      <xsl:for-each select="cda:value">
        <td valign="top">
          <!-- put out the appropriate NHSN label for the observation -->
          <xsl:call-template name="get-codeSystemName"/>
        </td>
        <td valign="top">
          <xsl:call-template name="get-displayName"/>
        </td>
      </xsl:for-each>
    </tr>
    <xsl:apply-templates select="*" mode="generate-narrative"/>
  </xsl:template>
  <!-- ============================================================== -->
  <!--                    OBSERVATION: ASSERTION PATTERN 3            -->
  <!-- ============================================================== -->
  <!--
       3. Assertion, create the label from templateId

          Label = map the templateId to a label
          Value = process value/@code
  -->
  <!-- context node is the clinical statement -->
  <xsl:template name="pattern-3">
    <tr>
      <td valign="top">
        <!-- put out the appropriate NHSN label for the observation -->
        <xsl:choose>
          <xsl:when test="cda:templateId/@root='2.16.840.1.113883.10.20.5.2.2.3'">BSI type</xsl:when>
          <xsl:when test="cda:templateId/@root='2.16.840.1.113883.10.20.5.2.2.3.1'">LCBI pathway</xsl:when>
          <xsl:when test="cda:templateId/@root='2.16.840.1.113883.10.20.5.2.2.7.4'">ASA class</xsl:when>
          <xsl:when test="cda:templateId/@root='2.16.840.1.113883.10.20.5.2.2.7.13'">SSI anatomic site</xsl:when>
          <xsl:when test="cda:templateId/@root='2.16.840.1.113883.10.20.5.2.5.1'">Microorganism identified</xsl:when>
          <xsl:when test="cda:templateId/@root='2.16.840.1.113883.10.20.5.6.4'">Bloodstream infection evidence type</xsl:when>
          <xsl:when test="cda:templateId/@root='2.16.840.1.113883.10.20.5.6.21'">Infection condition</xsl:when>
          <xsl:when test="cda:templateId/@root='2.16.840.1.113883.10.20.5.6.23'">Infection type</xsl:when>
          <xsl:when test="cda:templateId/@root='2.16.840.1.113883.10.20.5.6.27'">Occasion of HAI Detection</xsl:when>
          <xsl:when test="cda:templateId/@root='2.16.840.1.113883.10.20.5.6.49'">VIS type provided</xsl:when>
          <xsl:when test="cda:templateId/@root='2.16.840.1.113883.10.20.5.6.56'">History of Object Presence</xsl:when>
        </xsl:choose>
      </td>

      <td valign="top">
        <!-- change context node -->
        <xsl:for-each select="cda:value">
          <xsl:call-template name="get-displayName"/>
        </xsl:for-each>
      </td>
    </tr>
    <xsl:apply-templates select="*" mode="generate-narrative"/>
  </xsl:template>
  <!-- ============================================================== -->
  <!--                    QUESTION/ANSWER PATTERN 4                   -->
  <!-- ============================================================== -->
  <!--
         4. Question/answer pattern
  
            Label = process the code element
            Value = process value/@code, with some attention to @xsi:type 
  -->
  <!-- question/answer pattern: label from code, answer from value -->
  <!-- context node is the clinical statement -->
  <xsl:template name="pattern-4">
    <tr>
      <td valign="top">
        <!-- change context node -->
        <xsl:for-each select="cda:code">
          <xsl:call-template name="get-displayName"/>
        </xsl:for-each>
      </td>
      <td valign="top">
        <!-- change context node -->
        <xsl:for-each select="cda:value | cda:interpretationCode">
          <xsl:choose>
            <xsl:when test="@xsi:type='INT'">
              <xsl:value-of select="@value" />
            </xsl:when>
            <xsl:when test="@xsi:type='PQ'">
              <xsl:call-template name="get-quantity"/>
            </xsl:when>
            <xsl:when test="@xsi:type='IVL_TS'">
              <!-- change context node -->
              <xsl:for-each select="cda:width">
                <xsl:call-template name="get-quantity"/>
              </xsl:for-each>
            </xsl:when>
            <xsl:otherwise>
              <xsl:call-template name="get-displayName"/>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:for-each>
      </td>
    </tr>
    <xsl:apply-templates select="*" mode="generate-narrative"/>
  </xsl:template>
  <!-- ============================================================== -->
  <!--                    5. Code and negationInd                                                              -->
  <!-- ============================================================== -->
  <!--
       5. Code/Negation pattern
       
                 Label = process the code element
                 Value = process negationInd
  -->
  <!-- context node is the clinical statement -->
  <xsl:template name="pattern-5">
    <tr>
      <td valign="top">
        <xsl:for-each select="cda:code">
          <xsl:call-template name="get-displayName"/>
          <xsl:text> </xsl:text>
        </xsl:for-each>
      </td>
      <td valign="top">
        <xsl:call-template name="get-truthValue"/>
      </td>
    </tr>
  </xsl:template>
  
  <!-- ============================================================== -->
  <!--                    EXCEPTIONS MODE                             -->
  <!-- ============================================================== -->
  <!-- context node is the clinical statement -->
  
  <!-- Pattern Exception: For this encounter we must account for
       the child participant(s) if any. -->
  <xsl:template match="cda:encounter[cda:templateId/@root='2.16.840.1.113883.10.20.5.6.47']" 
                mode="exceptions">
    <xsl:apply-templates select="cda:participant" mode="generate-narrative"/>
    <xsl:apply-templates select="cda:entryRelationship" mode="generate-narrative" />
  </xsl:template>
  
  <!-- Pattern Exception: HAI Prior Discharge Encounter 2.16.840.1.113883.10.20.1.21
      Displays the encounter type and date of the encounter.
      Uses the get-displayName and format-date templates to format the data displayed. -->
  <xsl:template match="cda:encounter[cda:templateId/@root='2.16.840.1.113883.10.20.1.21']" mode="exceptions">
    <tr>
      <td valign="top">Encounter</td>
      <td valign="top">
        <xsl:for-each select="cda:code">
          <xsl:call-template name="get-displayName"/>
        </xsl:for-each>
      </td>
    </tr>
    <xsl:if test="cda:effectiveTime/cda:high">
      <tr>
        <td valign="top">Date</td>
        <td valign="top">
          <xsl:apply-templates select="cda:effectiveTime/cda:high/@value" mode="format-date"/>
        </td>
      </tr>
    </xsl:if>
  </xsl:template>
  
  <!-- Pattern Exception: Specimen Collection Procedure (LIO) 2.16.840.1.113883.10.20.5.6.53
    Displays the date and type of the specimen collected in the procedure.
    The format-date and get-displayName templates are re-used within this template to format the data displayed. -->
  <xsl:template match="cda:procedure[cda:templateId/@root='2.16.840.1.113883.10.20.5.6.53']" mode="exceptions">
    <tr>
      <td>Date specimen collected</td>
      <td>
        <xsl:apply-templates select="cda:effectiveTime/@value" mode="format-date"/>
      </td>
    </tr>
    <tr>
      <td>Specimen type</td>
      <td>
        <xsl:for-each select="cda:participant[@typeCode='PRD']/cda:participantRole/cda:playingEntity/cda:code">
          <xsl:call-template name="get-displayName"/>
        </xsl:for-each>
      </td>
    </tr>
    <xsl:apply-templates select="cda:entryRelationship" mode="generate-narrative" />
  </xsl:template>
  
  <!-- Pattern Exception: Specimen Collection Encounter (LIO) 2.16.840.1.113883.10.20.5.6.54
      Displays the in-facility location, date/time of admission/transfer and the playingEntity code/value for the specimen
      collection encounter. The get-codeSystemName, get-displayName, format-date and in-facility-location templates
      are re-used within this template to format the data displayed. -->
  <xsl:template match="cda:encounter[cda:templateId/@root='2.16.840.1.113883.10.20.5.6.54']" mode="exceptions">
    <tr>
      <td>In-facility location where patient was when specimen was drawn</td>
      <td>
        <!-- change context node -->
        <xsl:for-each select="cda:participant[@typeCode='LOC']/cda:participantRole[@classCode='SDLOC']">
          <xsl:if test=".//cda:id/@extension">
                <xsl:value-of select=".//cda:id/@extension"/>
                <xsl:text> </xsl:text>
              </xsl:if>
              <!-- change context node -->
              <xsl:for-each select=".//cda:code">
                <xsl:call-template name="get-displayName"/>
              </xsl:for-each>
        </xsl:for-each>
      </td>
    </tr>

    <tr>
      <td>Date admitted/transferred to the location where the specimen was drawn</td>
      <td>
        <xsl:apply-templates select="cda:effectiveTime/cda:low/@value" mode="format-date"/>
      </td>
    </tr>

  </xsl:template>
  
  <!-- Pattern Exception: pathogenRank, ...5.2.5.1.1
     has a table of values and thus is handled in this transform with the
     QA patterns. It is expected the CDA template will be revised in future
     to address this; when that happens, remove the exception template here. -->
  <xsl:template match="cda:observation[cda:templateId/@root='2.16.840.1.113883.10.20.5.2.5.1.1']" mode="exceptions">
    <tr>
      <td valign="top">Pathogen Ranking</td>
      <td valign="top">
        <!-- use value/@code here:
               - can't rely on the optional displayName being present
               - but there isn't a table of values either -->
        <xsl:value-of select="cda:value/@code"/>
      </td>
    </tr>
  </xsl:template>
  
  <!-- Pattern Exception: Infection Contributed to death, ...5.6.22
     This records whether the infection (id)
     contributed to the patient's death (entryRel/@typeCode=CAUS)
     There is no value element. -->
  <xsl:template match="cda:observation[cda:templateId/@root='2.16.840.1.113883.10.20.5.6.22']" mode="exceptions">
    <tr>
      <td valign="top">Infection Contributed to Death</td>
      <td valign="top">
        <!-- change context node -->
        <xsl:for-each select="..">
          <xsl:call-template name="get-truthValue"/>
        </xsl:for-each>
      </td>
    </tr>
  </xsl:template>
  
  <!-- Pattern Exception: Adverse Reactions ...5.6.2
       This is like pattern 1, simple negationInd,but here
       we stuff them all into a single table row, like the
       Criteria of Diagnosis. But here we don't have an
       organizer so we have to make the group ourselves. -->
  <xsl:template match="cda:observation[cda:templateId/@root='2.16.840.1.113883.10.20.5.6.2']" mode="exceptions">
    
    <xsl:if test="not(preceding::cda:observation[cda:templateId/@root='2.16.840.1.113883.10.20.5.6.2'])">
      <tr>
        <td valign="top">Adverse Reactions</td>
        <td valign="top">
          <!-- change context node to parent clinical statement -->
          <xsl:for-each select="../../cda:entryRelationship/cda:observation[cda:templateId/@root='2.16.840.1.113883.10.20.5.6.2']">
            <!-- change context node to its code -->
            <xsl:for-each select="cda:value">
              <xsl:call-template name="get-displayName"/>
            </xsl:for-each>
            <xsl:text>: </xsl:text>
            <xsl:call-template name="get-truthValue"/>
            <br/>
          </xsl:for-each>           
        </td>
      </tr>
    </xsl:if>
  </xsl:template>


  <!-- Pattern Exception: Hospital admission to ICU ...5.6.1
       This is like pattern 5, but here we have to process
       the child participant into the row label. -->
   <xsl:template match="cda:act[cda:templateId/@root='2.16.840.1.113883.10.20.5.6.1']" mode="exceptions">
    <tr>
      <td valign="top">
        <xsl:for-each select="cda:code">
          <xsl:call-template name="get-displayName"/>
        </xsl:for-each>
        <xsl:text> to </xsl:text>
        <!-- change context node -->
        <xsl:for-each select="cda:participant/cda:participantRole/cda:code">
          <xsl:call-template name="get-displayName"/>
        </xsl:for-each>
        <xsl:text> because of </xsl:text>
        <!-- change context node -->
        <xsl:for-each select="ancestor::cda:observation[1]/cda:value">
          <xsl:call-template name="get-displayName"/>
        </xsl:for-each>
      </td>
      <td valign="top">
        <xsl:call-template name="get-truthValue"/>
      </td>
    </tr>
   </xsl:template>
   
   <!-- Pattern Exception: hospital admission -for reason- of infection complications -->
   <!-- This is like pattern 1, negationInd, but here we have to process
        the reason - for reason of [infection type] complications -->
    <xsl:template match="cda:procedure[cda:templateId/@root='2.16.840.1.113883.10.20.5.6.6']" mode="exceptions">
      <tr>
        <td valign="top">
          <!-- change context node -->
          <xsl:for-each select="cda:code">
            <xsl:call-template name="get-displayName"/>
          </xsl:for-each>
          <xsl:text> because of </xsl:text>
          <!-- change context node -->
          <xsl:for-each select="ancestor::cda:observation[1]/cda:value">
            <xsl:call-template name="get-displayName"/>
          </xsl:for-each>
          <xsl:text> complications</xsl:text>
        </td>
        <td valign="top">
          <xsl:call-template name="get-truthValue"/>
        </td>
      </tr>  
    </xsl:template>
  
  <!-- Pattern Exception: simple substanceAdministration
          Anesthesia Administration .  ...5.2.2.7.3
          Antiseptic Ointment         ... 5.6.3
     This substanceAdministration is basically a simple assertion w/negationInd,
     pattern 1, but the label is in the code element, not the value element, 
     and is several levels down. -->
  <xsl:template match="*[cda:templateId/@root='2.16.840.1.113883.10.20.5.2.2.7.3'
                      or cda:templateId/@root='2.16.840.1.113883.10.20.5.6.3']" mode="exceptions">
    <tr>
      <td valign="top">
        <!-- change context node -->
        <xsl:for-each select="cda:consumable/cda:manufacturedProduct/cda:manufacturedLabeledDrug/cda:code">
          <xsl:call-template name="get-displayName"/>
        </xsl:for-each>
      </td>
      <td valign="top">
        <xsl:call-template name="get-truthValue"/>
      </td>
    </tr>
    <xsl:apply-templates select="*" mode="generate-narrative"/>
  </xsl:template>
  
  <!-- Pattern Exceptions: 
              Procedure Details in a Procedure Report     ...5.6.33 
              Procedure Details in an SSI Report          ...5.6.34 
              Procedure Details in a CLIP Report          ...5.6.32
              Risk Factors in a Procedure Report          ...5.6.35 

     These are all procedure elements. They may contain code, effectiveTime,
     methodCode, and approachSiteCode elements which are handled in the
     "CDA Non-templated elements" section of this transform, as well as
     observations.
     All the procedure needs to do is process its children in the usual way.
     This template is needed because procedure has a templateId, and so the
     transform will kick out a diagnostic unless they're handled explicitly. -->
  <xsl:template match="*[cda:templateId/@root='2.16.840.1.113883.10.20.5.6.33'
                      or cda:templateId/@root='2.16.840.1.113883.10.20.5.6.34'
                      or cda:templateId/@root='2.16.840.1.113883.10.20.5.6.32'
                      or cda:templateId/@root='2.16.840.1.113883.10.20.5.6.35']" mode="exceptions">
    <xsl:apply-templates select="*" mode="generate-narrative"/>
  </xsl:template>
  
  <!-- Pattern Exception: CLIP Protocol      ...5.6.7 -->
  <!-- The CLI Protocol is represented as a set of list items.
     Each produces a table of its entries. -->
  <xsl:template match="*[cda:templateId/@root='2.16.840.1.113883.10.20.5.6.7']" mode="exceptions">
    <paragraph>
      <content>This section records whether the steps were 
    performed in the sequence shown.</content>
    </paragraph>
    <list listType="ordered">
      <xsl:apply-templates mode="exceptions"/>
    </list>
  </xsl:template>
  
  <!-- Pattern Exception: Protocol step 1    
     Contains ...20.5.6.8 
         (which contains ...5.6.16  ...5.6.43 ...5.6.45 ...5.6.42 ) -->
  <xsl:template match="cda:entryRelationship[cda:sequenceNumber/@value='1']" mode="exceptions">
    <item>Preparation<br/>
      <table width="100%">
        <tbody>
          <xsl:apply-templates select="*" mode="generate-narrative"/>
        </tbody>
      </table>
    </item>
  </xsl:template>
  
  <!-- Pattern Exception: Protocol step 2 
     Contains ...20.5.6.44 -->
  <xsl:template match="cda:entryRelationship[cda:sequenceNumber/@value='2']" mode="exceptions">
    <item>Solutions Dried<br/>
      <table width="100%">
        <tbody>
          <xsl:apply-templates select="*" mode="generate-narrative"/>
        </tbody>
      </table>
    </item>
  </xsl:template>
  <!-- Pattern Exception: Protocol step 3
     Contains ...20.5.6.36 (which contains participant and upd-2016, upd-2017) -->
  <xsl:template match="cda:entryRelationship[cda:sequenceNumber/@value='3']" mode="exceptions">
    <item>CLI Procedure<br/>
      <table width="100%">
        <tbody>
          <xsl:apply-templates select="*" mode="generate-narrative"/>
        </tbody>
      </table>
    </item>
  </xsl:template>
  <!-- Pattern Exception: Protocol step 4
     Contains ...20.5.6.3 -->
  <xsl:template match="cda:entryRelationship[cda:sequenceNumber/@value='4']" mode="exceptions">
    <item>Antiseptic Ointment<br/>
      <table width="100%">
        <tbody>
          <xsl:apply-templates select="*" mode="generate-narrative"/>
        </tbody>
      </table>
    </item>
  </xsl:template>
  <!-- Pattern Exception: CLI Procedure  ...20.5.6.36 -->
  <xsl:template match="*[cda:templateId/@root='2.16.840.1.113883.10.20.5.6.36']" mode="exceptions">
    <!-- the heading's already been created by the handling of the sequence item -->
    <!-- processes the participant, which is templated but ought not to be -->
    <xsl:apply-templates select="*" mode="generate-narrative"/>
  </xsl:template>
  
  <!-- Pattern Exception: Skin Preparation Solutions Applied
          part of step 1 within CLI protocol.                           ...20.5.6.43 -->
  <!-- 2012 is an organizer, it lists several ...20.5.6.42 skin-prep solutions as yes/no -->
  <xsl:template match="*[cda:templateId/@root='2.16.840.1.113883.10.20.5.6.43']" mode="exceptions">
    <tr>
      <td valign="top">Skin preparation solutions</td>
      <td valign="top">
        <xsl:for-each select="cda:component/cda:substanceAdministration">
          <!-- change context node -->
          <xsl:for-each select="cda:consumable/cda:manufacturedProduct/cda:manufacturedMaterial/cda:code">
            <xsl:call-template name="get-displayName"/>
          </xsl:for-each>
          <xsl:text>: </xsl:text>
          <xsl:call-template name="get-truthValue"/>
          <xsl:if test="../following-sibling::cda:component/cda:substanceAdministration">
            <br/>
          </xsl:if>
        </xsl:for-each>
      </td>
    </tr>
    <xsl:apply-templates select="*" mode="generate-narrative"/>
  </xsl:template>
  <!-- Pattern Exception: Sterile Barriers       ...20.5.6.45 -->
  <xsl:template match="*[cda:templateId/@root='2.16.840.1.113883.10.20.5.6.45']" mode="exceptions">
    <tr>
      <td valign="top">
        <xsl:for-each select="cda:code">
          <xsl:call-template name="get-displayName"/>
        </xsl:for-each>
      </td>
      <!-- make only one cell, i.e. for the first participant -->
      <!-- also changes context node -->
      <xsl:for-each select="cda:participant[1]">
        <td>
          <!-- change context node -->
          <xsl:for-each select="cda:participantRole/cda:playingDevice/cda:code">
            <xsl:call-template name="get-displayName"/>
          </xsl:for-each>
          <!-- get any other participants -->
          <xsl:for-each select="following-sibling::cda:participant">
            <br/>
            <!-- change context node -->
            <xsl:for-each select="cda:participantRole/cda:playingDevice/cda:code">
              <xsl:call-template name="get-displayName"/>
            </xsl:for-each>
          </xsl:for-each>
        </td>
      </xsl:for-each>
    </tr>
    <xsl:apply-templates select="*[not(self::cda:code) and not(self::cda:participant)]" mode="generate-narrative"/>
  </xsl:template>
  
  <!-- Pattern Exception: Reason for Procedure       ...20.5.6.38 -->
  <!-- The entryRel is @typeCode=RSON, we need to provide the label -->
  <xsl:template match="*[cda:templateId/@root='2.16.840.1.113883.10.20.5.6.38']" mode="exceptions">
    <tr>
      <td valign="top">Reason for procedure</td>
      <td valign="top">
        <xsl:for-each select="cda:value">
          <xsl:call-template name="get-displayName"/>
        </xsl:for-each>
      </td>
    </tr>
  </xsl:template>
  
  <!-- Pattern Exception: Vaccination Offer      ...20.5.6.18 -->
  <xsl:template match="*[cda:templateId/@root='2.16.840.1.113883.10.20.5.6.18']" mode="exceptions">

    <!-- first, the offer itself -->
    <tr>
      <td valign="top">
        <xsl:for-each select="cda:code">
          <xsl:call-template name="get-displayName"/>
          <xsl:text> offered</xsl:text>
        </xsl:for-each>
      </td>
      <td>
        <xsl:call-template name="get-truthValue"/>
      </td>
    </tr>

    <!-- at least one Eligibility Criterion is required, and there may be many.
         They all go into a single, right-hand cell. 
         This handles templateId ...5.6.13 -->
    <tr>
      <td valign="top">Eligibility criteria</td>
      <td>
        <xsl:for-each select="cda:entryRelationship[@typeCode='RSON']/*[cda:templateId/@root='2.16.840.1.113883.10.20.5.6.13']">
          
          <!-- change context node to process the value -->
          <xsl:for-each select="cda:value">
            <xsl:call-template name="get-displayName"/>
          </xsl:for-each>
          
          
          <!-- Tricky stuff here. If it's an HCW, there's only one criterion
               and it has participants about the person's employment and
               other information.
               But if it's not an HCW, there are a lot of criteria.
               In either case, it all goes into the right-hand cell. -->
          <xsl:choose>
            <!-- if HCW -->
            <xsl:when test="cda:participant">
              <!-- change context node -->
              <xsl:for-each select="cda:participant[@typeCode='SBJ']/cda:participantRole/cda:code">
                <xsl:text> as </xsl:text>
                <xsl:call-template name="get-displayName"/>
              </xsl:for-each>
              <xsl:text> (</xsl:text>
              <!-- the location type -->
              <xsl:for-each select="cda:participant[@typeCode='LOC']/cda:participantRole/cda:code">
                <xsl:call-template name="get-displayName"/>
                <xsl:text>)</xsl:text>
                <br/>
              </xsl:for-each>
              
              <!-- also, while we're here in 5.6.13 in an HCW, pick up the children.
                   These are: 5.6.30, and 5.6.28 which in turn contains 5.6.9 -->
              <xsl:apply-templates select="cda:entryRelationship/*" mode="data-only"/>
            </xsl:when>

            <!-- otherwise the condition is a criterion of the patient:
                 there won't be any participants. Instead we put display
                 the truthValue of the condition -->
            <xsl:otherwise>
                <xsl:text>: </xsl:text>
                <xsl:call-template name="get-truthValue"/> 
            </xsl:otherwise>

          </xsl:choose> 
          
          <!-- if there's another criterion after this, process it into the same table cell -->
          <xsl:if test="../following-sibling::cda:entryRelationship/*[cda:templateId/@root='2.16.840.1.113883.10.20.5.6.13']">
            <br/>
          </xsl:if>
          
        </xsl:for-each>
      </td>
    </tr>
         
    <xsl:apply-templates select="*" mode="generate-narrative"/>
  </xsl:template>
  
  <!-- Pattern Exception: Performs direct patient care ...5.6.30.
       This is tucked into a right-hand cell, does not create a row of its own -->
  <xsl:template match="*[cda:templateId/@root='2.16.840.1.113883.10.20.5.6.30']" mode="data-only">
    <xsl:for-each select="cda:value">
      <xsl:call-template name="get-displayName"/>
    </xsl:for-each>
    <xsl:text>: </xsl:text>
    <xsl:call-template name="get-truthValue"/>
    <br/>
  </xsl:template>
  
  <!-- Pattern Exception: HCW occupation ...5.6.28
       This is tucked into a right-hand cell, does not create a row of its own.
       It does have a child that we have to process. -->
  <xsl:template match="*[cda:templateId/@root='2.16.840.1.113883.10.20.5.6.28']" mode="data-only">
    <xsl:for-each select="cda:code">
      <xsl:call-template name="get-displayName"/>
    </xsl:for-each>
    <xsl:text>: </xsl:text>
    <xsl:for-each select="cda:value">
      <xsl:call-template name="get-displayName"/>
    </xsl:for-each>
    <br/>
    <xsl:apply-templates select="*" mode="data-only"/>
  </xsl:template>
  
  <!-- Pattern Exception: HCW clinical specialty ...5.6.9
       This is tucked into a right-hand cell, does not create a row of its own -->
  <xsl:template match="*[cda:templateId/@root='2.16.840.1.113883.10.20.5.6.9']" mode="data-only">
    <xsl:for-each select="cda:code">
      <xsl:call-template name="get-displayName"/>
    </xsl:for-each>
    <xsl:text>: </xsl:text>
    <xsl:for-each select="cda:value">
      <xsl:call-template name="get-displayName"/>
    </xsl:for-each>
  </xsl:template>

  <!-- Pattern Exception: Vaccination       ...20.5.6.17 -->
  <xsl:template match="*[cda:templateId/@root='2.16.840.1.113883.10.20.5.6.17']" mode="exceptions">
    <tr>
      <td valign="top">
        <!-- change context node -->
        <xsl:for-each select="cda:code">
          <xsl:call-template name="get-displayName"/>
        </xsl:for-each>
        <xsl:text> administered</xsl:text>
      </td>
      <td>
        <xsl:call-template name="get-truthValue"/>
      </td>
    </tr>
    <xsl:apply-templates select="*" mode="generate-narrative"/>
  </xsl:template>
  
  <!-- Pattern Exception: Seasons Immunized      ...20.5.6.40 -->
  <!-- The only datum is the effectiveTime -->
  <xsl:template match="*[cda:templateId/@root='2.16.840.1.113883.10.20.5.6.40']" mode="exceptions">
    <xsl:apply-templates select="*" mode="generate-narrative"/>
  </xsl:template>
  
  <!-- Pattern Exception: Criteria of Diagnosis Organizer      ...20.5.6.11 -->
  <xsl:template match="*[cda:templateId/@root='2.16.840.1.113883.10.20.5.6.11']" mode="exceptions">
    <tr>
      <td valign="top">Criteria of diagnosis</td>
      <td>
        <xsl:apply-templates select="cda:component/cda:observation[cda:templateId/@root='2.16.840.1.113883.10.20.5.6.10']" mode="exceptions"/>
      </td>
    </tr>
    <!-- don't need to process children, we've taken care of them -->
  </xsl:template>

  <!-- Pattern Exception: Criterion of Diagnosis      ...20.5.6.10 -->
  <!-- These go in 2nd cell of a row -->
  <xsl:template match="*[cda:templateId/@root='2.16.840.1.113883.10.20.5.6.10']" mode="exceptions">
    <!-- change context node -->
    <xsl:for-each select="cda:value">
      <xsl:call-template name="get-displayName"/>
    </xsl:for-each>
    <xsl:text>: </xsl:text>
    <xsl:call-template name="get-truthValue"/>
    <xsl:if test="following::*[cda:templateId/@root='2.16.840.1.113883.10.20.5.6.10']">
      <br/>
    </xsl:if>
    <!-- don't need to process children, these don't have any -->
  </xsl:template>

  <!-- Pattern Exception - Significant Pathogens  ... 20.5.6.41 -->
  <!-- This is a Q/A pattern but has more than one value element to display -->
  <xsl:template match="*[cda:templateId/@root='2.16.840.1.113883.10.20.5.6.41']" mode="exceptions">
    <tr>
      <td valign="top">Significant Pathogens</td>
      <td>
        <!-- change context node -->
        <xsl:for-each select="cda:value">
          <xsl:call-template name="get-displayName"/>
          <xsl:if test="following-sibling::cda:value">
            <xsl:text>, </xsl:text>
          </xsl:if>
        </xsl:for-each>
      </td>
    </tr>
    <!-- don't need to process children, there aren't any -->
  </xsl:template>
  
  <!-- ============================================================== -->
  <!-- ============================================================== -->
  <!--                    NON-TEMPLATED CDA ELEMENTS SEGMENT          -->
  <!-- ============================================================== -->
  <!-- ============================================================== -->
  <!-- procedure code: default handling -->
  <xsl:template match="cda:procedure/cda:code" mode="generate-narrative">
    <tr>
      <td valign="top">Type of procedure</td>
      <td valign="top">
        <xsl:call-template name="get-displayName"/>
      </td>
    </tr>
  </xsl:template>
  
  <!-- procedure date, low only -->
  <xsl:template match="cda:procedure/cda:effectiveTime/cda:low" mode="generate-narrative">
    <tr>
      <td valign="top">Procedure date</td>
      <td valign="top">
        <xsl:apply-templates select="@value" mode="format-date"/>
      </td>
    </tr>
  </xsl:template>
  
  <!-- procedure date, @value -->
  <xsl:template match="cda:procedure/cda:effectiveTime[not(cda:low) and not(cda:width)]" mode="generate-narrative">
    <tr>
      <td valign="top">Procedure date</td>
      <td valign="top">
        <xsl:apply-templates select="@value" mode="format-date"/>
      </td>
    </tr>
  </xsl:template>
  
  <!-- procedure duration, default -->
  <xsl:template match="cda:procedure/cda:effectiveTime/cda:width" mode="generate-narrative">
    <tr>
      <td valign="top">Procedure duration</td>
      <td valign="top">
        <xsl:call-template name="get-quantity"/>
      </td>
    </tr>
  </xsl:template>
  
  <!-- procedure duration in a Procedure Report:
       also displays the admission status, from the header -->
  <xsl:template match="cda:procedure/cda:effectiveTime/cda:width
          [/cda:ClinicalDocument/cda:templateId/@root='2.16.840.1.113883.10.20.5.3.4']" mode="generate-narrative" priority="2">
    <tr>
      <td valign="top">Procedure duration</td>
      <td valign="top">
        <xsl:call-template name="get-quantity"/>
      </td>
    </tr>
    <!-- admission status -->
    <tr>
      <td valign="top">Admission status</td>
      <td valign="top">
        <!-- change context node -->
        <xsl:for-each select="/cda:ClinicalDocument/cda:componentOf/cda:encompassingEncounter
                                 /cda:code">
          <xsl:call-template name="get-displayName"/>
        </xsl:for-each>
      </td>
    </tr>
  </xsl:template>

  <!-- effectiveTime in an Infection Type Observation ...5.6.23 -->
  <xsl:template match="cda:effectiveTime[../cda:templateId/@root='2.16.840.1.113883.10.20.5.6.23']" mode="generate-narrative">
    <tr>
      <td valign="top">Date of infection</td>
      <td valign="top">
        <xsl:apply-templates select="@value" mode="format-date"/>
      </td>
    </tr>
  </xsl:template>
  
  <!-- effectiveTime in an Occasion of HAI Detection Observation ...5.2.2.5 -->
  <xsl:template match="cda:effectiveTime
                      [../cda:templateId/@root='2.16.840.1.113883.10.20.5.2.2.5']" mode="generate-narrative">
    <tr>
      <td valign="top">Date of infection</td>
      <td valign="top">
        <xsl:apply-templates select="@value" mode="format-date"/>
      </td>
    </tr>
  </xsl:template>
  
  <!-- effectiveTime in Seasons Immunized Obs -->
  <!-- the values are 4-digit years, the format-date mode fails on these -->
  <xsl:template match="cda:effectiveTime
                      [../cda:templateId/@root='2.16.840.1.113883.10.20.5.6.40']" mode="generate-narrative">
    <tr>
      <td valign="top">Seasons Immunized</td>
      <td valign="top">
        <xsl:value-of select="cda:low/@value"/>
        <xsl:text>-</xsl:text>
        <xsl:value-of select="cda:high/@value"/>
      </td>
    </tr>
  </xsl:template>
  
  <!-- effectiveTime as Date Administered in an Immunization -->
  <xsl:template match="cda:effectiveTime
                       [../cda:templateId/@root='2.16.840.1.113883.10.20.5.6.17']" mode="generate-narrative">
    <tr>
      <td valign="top">Date of immunization</td>
      <td valign="top">
        <xsl:apply-templates select="@value" mode="format-date"/>
      </td>
    </tr>
  </xsl:template>
  
  <!-- routeCode in an immunization -->
  <xsl:template match="cda:routeCode[../cda:templateId/@root='2.16.840.1.113883.10.20.5.6.17']" mode="generate-narrative">
    <tr>
      <td valign="top">Route of administration</td>
      <td valign="top">
        <xsl:call-template name="get-displayName"/>
      </td>
    </tr>
    <xsl:apply-templates select="*" mode="generate-narrative"/>
  </xsl:template>
  
  <!-- consumable in an immunization -->
  <xsl:template match="cda:consumable[../cda:templateId/@root='2.16.840.1.113883.10.20.5.6.17']" mode="generate-narrative">
    <tr>
      <td valign="top">Product administered</td>
      <td valign="top">
        <!-- change context node -->
        <xsl:for-each select="cda:manufacturedProduct/cda:manufacturedMaterial/cda:code">
          <xsl:call-template name="get-displayName"/>
          <xsl:if test="following-sibling::cda:lotNumberText">
            <xsl:text> (lot number </xsl:text>
            <xsl:apply-templates select="following-sibling::cda:lotNumberText"/>
            <xsl:text>)</xsl:text>
          </xsl:if>
        </xsl:for-each>
      </td>
    </tr>
    <xsl:apply-templates select="*" mode="generate-narrative"/>
  </xsl:template>
  
  <!-- targetSiteCode: show label and displayName, and
       process any children in the usual way. -->
  <xsl:template match="cda:targetSiteCode" mode="generate-narrative">
    <tr>
      <td valign="top">Procedure site</td>
      <td valign="top">
        <xsl:call-template name="get-displayName"/>
      </td>
    </tr>
    <xsl:apply-templates select="*" mode="generate-narrative"/>
  </xsl:template>
  
  <!-- methodCode and approachSiteCode: default handling = 
       show label derived from valueSetName, and code -->
  <xsl:template match="cda:methodCode | cda:approachSiteCode" mode="generate-narrative">
    <tr>
      <!-- first cell is the label -->
      <td valign="top">
        <xsl:choose>
          <!-- if it's in hai_voc.xml -->
          <xsl:when test="$the-valuesets/*/*[@root='2.16.840.1.113883.6.277']
                                            [*[@value=current()/@code]]">
            <xsl:for-each select="$the-valuesets/*/*[@root='2.16.840.1.113883.6.277']
                                                [*[@value=current()/@code]]">
              <xsl:value-of select="@valueSetName"/>
           </xsl:for-each>
          </xsl:when>
          
          <!-- if not, perhaps hard-code the label using the templateId -->
          <!--
          <xsl:when test="">
          
          </xsl:when>
          -->
          
          <!-- otherwise we don't know much about it, 
               the best we can do is put out the codeSystemName and OID -->
          <xsl:otherwise>
            <xsl:value-of select="@codeSystemName"/>
            <xsl:text> </xsl:text>
            <xsl:value-of select="@codeSystem"/>
          </xsl:otherwise>
        </xsl:choose>
      </td>

      <!-- second cell is the value -->
      <td valign="top">
        <xsl:call-template name="get-displayName"/>
      </td>
    </tr>
  </xsl:template>



<!-- KK - change this to a simple lookup of the code in voc.xml,
           and use negationInd for the truthValue -->

  <!-- methodCode: exception handling for emergency -->
  <xsl:template match="cda:methodCode[@codeSystem='2.16.840.1.113883.6.96']
                                     [@code='373110003']" mode="generate-narrative">
    <tr>
      <td valign="top">Emergency procedure</td>
      <td valign="top">Yes</td>
    </tr>
  </xsl:template>
    
  
  <!-- participant in a CLIP procedure - the catheter type -->
  <xsl:template match="cda:participant[../cda:templateId/@root='2.16.840.1.113883.10.20.5.6.36']" mode="generate-narrative">
    <tr>
      <td valign="top">Catheter type</td>
      <td valign="top">
        <!-- change context node -->
        <xsl:for-each select="cda:participantRole/cda:playingDevice/cda:code">
          <xsl:call-template name="get-displayName"/>
        </xsl:for-each>
      </td>
    </tr>
  </xsl:template>
  
  <!-- location participant in an immunization of healthcare worker -->
  <xsl:template match="cda:participant[@typeCode='LOC'][../cda:templateId/@root='2.16.840.1.113883.10.20.5.6.17']" mode="generate-narrative">
    <tr>
      <td valign="top">Location administered</td>
      <td valign="top">
        <!-- change context node -->
        <xsl:for-each select="cda:participantRole/cda:code">
          <xsl:call-template name="get-displayName"/>
        </xsl:for-each>
      </td>
    </tr>

  </xsl:template>
  
  <!-- participant in a pop-sum encounter:
       either in-facility location or definition of sub-group -->
  <xsl:template match="cda:participant[../cda:templateId/@root='2.16.840.1.113883.10.20.5.6.47']" mode="generate-narrative">
    <xsl:choose>
      <xsl:when test="@typeCode='LOC'">
        <tr>
          <td><content styleCode="Bold">In-facility Location</content></td>
          <td><xsl:if test=".//cda:id/@extension">
                <xsl:value-of select=".//cda:id/@extension"/>
                <xsl:text> </xsl:text>
              </xsl:if>
              <!-- change context node -->
              <xsl:for-each select=".//cda:code">
                <xsl:call-template name="get-displayName"/>
              </xsl:for-each>
          </td>
        </tr>
      </xsl:when>
      <xsl:when test="@typeCode='SBJ'">
        <tr>
          <td><content styleCode="Bold">Sub-group</content></td>
          <!-- change context node -->
          <td><xsl:for-each select=".//cda:code">
                <xsl:call-template name="get-displayName"/>
              </xsl:for-each>
          </td>
        </tr>      
      </xsl:when>
      <xsl:otherwise>
        <tr>
          <td><content styleCode="Bold">[not labeled]</content></td>
          <!-- change context node -->
          <td><xsl:for-each select=".//cda:code">
                <xsl:call-template name="get-displayName"/>
              </xsl:for-each>
          </td>
        </tr>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>


  <!-- performer in CLIP procedure -->
  <xsl:template match="cda:performer[../cda:templateId/@root='2.16.840.1.113883.10.20.5.6.32']" mode="generate-narrative">
    <tr>
      <td valign="top">Role of performer</td>
      <td valign="top">
        <!-- change context node -->
        <xsl:for-each select="cda:assignedEntity/cda:code">
          <xsl:call-template name="get-displayName"/>
        </xsl:for-each>
      </td>
    </tr>
  </xsl:template>
  
  <!-- performer in immunization -->
  <xsl:template match="cda:performer[../cda:templateId/@root='2.16.840.1.113883.10.20.5.6.17']" mode="generate-narrative">
    <tr>
      <td valign="top">Person administering immunization</td>
      <td valign="top">
        <xsl:apply-templates select="cda:assignedEntity/cda:assignedPerson/cda:name" mode="display-person-name"/>
        <xsl:if test="cda:assignedEntity/cda:id">
          <br/>
          <xsl:apply-templates select="cda:assignedEntity/cda:id" mode="display-immunizer-id"/>
        </xsl:if>
      </td>
    </tr>
  </xsl:template>
  
  <xsl:template match="cda:name" mode="display-person-name">
    <xsl:for-each select="cda:given">
      <xsl:apply-templates select="."/>
      <xsl:text> </xsl:text>
    </xsl:for-each>
    <xsl:apply-templates select="cda:family"/>
    <xsl:if test="cda:prefix">
      <xsl:text>, </xsl:text>
      <xsl:apply-templates select="cda:prefix"/>
    </xsl:if>
  </xsl:template>
  
  <xsl:template match="cda:id" mode="display-immunizer-id">
    <xsl:text>ID </xsl:text>
    <xsl:value-of select="@extension"/>
    <xsl:text> (Facility </xsl:text>
    <xsl:value-of select="@root"/>
    <xsl:text>)</xsl:text>
  </xsl:template>
  
  <!-- ============================================================== -->
  <!-- ============================================================== -->
  <!--                    UTILITIES SEGMENT                           -->
  <!-- ============================================================== -->
  <!-- ============================================================== -->
  <!-- ============================================================== -->
  <!--                    CDA UTILITIES                               -->
  <!-- ============================================================== -->
  <!-- This transform displays the CDA document's value for
     displayName and codeSystemName if present, and otherwise
     picks up a value from voc.xml.

     The structure of voc.xml is:

<systems>

  <system root="2.16.840.1.113883.5.4"
          codeSystemName="HL7 actCode"
          valueSetName="NHSNEncounterTypeCode"
          valueSetOid="2.16.840.1.113883.13.1">
  <code value="AMB" displayName="Ambulatory"/>
  <code value="INP" displayName="Inpatient"/>
  </system>

  <system>...</system>
  ...
</systems>

-->

<!--
      <xsl:when test="@codeSystem='2.16.840.1.113883.6.277'">
        <xsl:for-each select="$the-valuesets/*/*[@root='2.16.840.1.113883.6.277']
                                            [*[@value=current()/@code]]">
          <xsl:value-of select="@valueSetName"/>
       </xsl:for-each>
-->

  <!-- context node is a code element or similar such as methodCode -->
  <xsl:template name="get-displayName">
  
    <xsl:variable name="this-code">
      <xsl:value-of select="@code"/>
    </xsl:variable>
    <xsl:variable name="this-codeSystem">
      <xsl:value-of select="@codeSystem"/>
    </xsl:variable>
    
    <xsl:choose>
      <!-- check in the value sets -->
      <xsl:when test="$the-valuesets/*/*[@root=$this-codeSystem]/*[@value=$this-code]">
        <xsl:value-of select="$the-valuesets/*/*[@root=$this-codeSystem]/*[@value=$this-code]/@displayName"/>
      </xsl:when>
      
      <!-- check in the NHSN SVB -->
      <xsl:when test="$the-valuesets/*/*[@root='2.16.840.1.113883.6.277'][@valueSetOID='']/*[@value=$this-code]">
        <xsl:value-of select="$the-valuesets/*/*[@root='2.16.840.1.113883.6.277'][@valueSetOID='']/*[@value=$this-code]/@displayName"/>
      </xsl:when>
      
      <!-- check in the "root overrides", i.e., the non-NHSN SVB -->
      <xsl:when test="$the-valuesets/*/*[@root=''][@valueSetOID='']/*[@value=$this-code][@rootOverride]">
        <xsl:value-of select="$the-valuesets/*/*[@root=''][@valueSetOID='']/*[@value=$this-code]/@displayName"/>
      </xsl:when>

      <!-- use the file's displayName if present -->
      <xsl:when test="@displayName">
        <xsl:value-of select="@displayName"/>
      </xsl:when>
      
      <!-- failing all else, put out the code and its codeSystem -->          
      <xsl:otherwise>
        <xsl:value-of select="@code"/>
        <xsl:text> / </xsl:text>
        <xsl:choose>
          <xsl:when test="@codeSystem='2.16.840.1.113883.6.96'">SNOMED</xsl:when>
          <xsl:when test="@codeSystem='2.16.840.1.113883.6.1'">LOINC</xsl:when>
          <xsl:when test="@codeSystem='2.16.840.1.113883.5.4'">HL7</xsl:when>
          <xsl:when test="@codeSystem='2.16.840.1.113883.6.277'">cdcNHSN</xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="@codeSystem"/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:otherwise>
    </xsl:choose>
    
  </xsl:template>
  

  <xsl:template name="get-codeSystemName">
  
    <xsl:choose>
      <!-- if it's the big NHSN codeSystem, get the valueSetName from voc.xml -->
      <xsl:when test="@codeSystem='2.16.840.1.113883.6.277'">
        <xsl:for-each select="$the-valuesets/*/*[@root='2.16.840.1.113883.6.277']
                                            [*[@value=current()/@code]]">
          <xsl:value-of select="@valueSetName"/>
       </xsl:for-each>
      </xsl:when>
      
      <!-- If not, then perhaps hardcode a label from the templateId -->
      <!-- ... -->
      
      <!-- If not, show the codeSystemName if present -->
      <xsl:when test="@codeSystemName">
        <xsl:value-of select="@codeSystemName"/>
      </xsl:when>
      
      <!-- If not, show the codeSystem OID -->
      <xsl:otherwise>
        <xsl:value-of select="@codeSystem"/>
      </xsl:otherwise>
    </xsl:choose>
    
  </xsl:template>
  

  <xsl:template name="get-truthValue">
    <xsl:choose>
      <xsl:when test="@negationInd='false'">Yes</xsl:when>
      <xsl:when test="@negationInd='true'">No</xsl:when>
    </xsl:choose>
  </xsl:template>
  
  
  <xsl:template name="get-quantity">
    <xsl:value-of select="@value"/>
    <xsl:text> </xsl:text>
    <!-- specify the unit too, except in a Population Summary Report,
       where the label already does that.-->
    <xsl:if test="not(/cda:ClinicalDocument/cda:templateId/@root='2.16.840.1.113883.10.20.5.15')">
      <xsl:value-of select="@unit"/>
    </xsl:if>
  </xsl:template>
  
  <!-- ============================================================== -->
  <!--                    GENERAL UTILITIES                           -->
  <!-- ============================================================== -->
  <!--  mode = format-date
      outputs a date in Month Day, Year form
      e.g., 19991207  ==> December 07, 1999
-->
  <xsl:template match="* | @*" name="format-date" mode="format-date">
    <!-- may be passed in by a calling template -->
    <xsl:param name="the-date"/>
    <xsl:param name="date">
      <!-- turn it into a string -->
      <xsl:choose>
        <xsl:when test="$the-date">
          <xsl:value-of select="$the-date"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="."/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:param>
    <!-- get and translate the month -->
    <xsl:param name="month" select="substring($date, 5, 2)"/>
    <!-- get month - a separate template so others can use it -->
    <xsl:call-template name="format-month">
      <xsl:with-param name="month" select="$month"/>
    </xsl:call-template>
    <!-- get the day -->
    <xsl:choose>
      <xsl:when test='substring ($date, 7, 1)="0"'>
        <xsl:value-of select="substring ($date, 8, 1)"/>
        <xsl:text>, </xsl:text>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="substring ($date, 7, 2)"/>
        <xsl:text>, </xsl:text>
      </xsl:otherwise>
    </xsl:choose>
    <!-- get the year -->
    <xsl:value-of select="substring ($date, 1, 4)"/>
  </xsl:template>
  <xsl:template name="format-month">
    <!-- catch from the calling template -->
    <xsl:param name="month"/>
    <xsl:choose>
      <xsl:when test="$month='01'">
        <xsl:text>January </xsl:text>
      </xsl:when>
      <xsl:when test="$month='02'">
        <xsl:text>February </xsl:text>
      </xsl:when>
      <xsl:when test="$month='03'">
        <xsl:text>March </xsl:text>
      </xsl:when>
      <xsl:when test="$month='04'">
        <xsl:text>April </xsl:text>
      </xsl:when>
      <xsl:when test="$month='05'">
        <xsl:text>May </xsl:text>
      </xsl:when>
      <xsl:when test="$month='06'">
        <xsl:text>June </xsl:text>
      </xsl:when>
      <xsl:when test="$month='07'">
        <xsl:text>July </xsl:text>
      </xsl:when>
      <xsl:when test="$month='08'">
        <xsl:text>August </xsl:text>
      </xsl:when>
      <xsl:when test="$month='09'">
        <xsl:text>September </xsl:text>
      </xsl:when>
      <xsl:when test="$month='10'">
        <xsl:text>October </xsl:text>
      </xsl:when>
      <xsl:when test="$month='11'">
        <xsl:text>November </xsl:text>
      </xsl:when>
      <xsl:when test="$month='12'">
        <xsl:text>December </xsl:text>
      </xsl:when>
      <xsl:otherwise>
        <xsl:text>uncatalogued month number</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
</xsl:transform>
